;; Generated package description from tramp.el  -*- no-byte-compile: t -*-
(define-package "tramp" "0.0.20201222.135152" "Transparent Remote Access, Multiple Protocol" '((emacs "25.1")) :keywords '("comm" "processes") :authors '(("Kai Großjohann" . "kai.grossjohann@gmx.net")) :maintainer '("Michael Albinus" . "michael.albinus@gmx.de") :url "https://www.gnu.org/software/tramp/")
